use std::{env, thread::sleep, time::{Duration, UNIX_EPOCH}};

use anyhow::{bail, Ok, Result};
use embedded_svc::mqtt::client::{Publish, QoS};
use esp_idf_sys as _;
use log::*;

mod message_queue;
mod wifi;
mod time;

use crate::{message_queue::MessageQueue, wifi::Internet, time::Time};
fn main() {
    env::set_var("DEFMT_LOG", "trace");
    env::set_var("RUST_BACKTRACE", "1");
    env::set_var("RUST_LOG", "trace");
    env_logger::init();

    // Temporary. Will disappear once ESP-IDF 4.4 is released, but for now it is necessary to call this function once,
    // or else some patches to the runtime implemented by esp-idf-sys might not link properly.
    esp_idf_sys::link_patches();

    println!("Hello, world!");

    let wifi = Internet::new().unwrap();

    let mut mq = MessageQueue::new();
    mq.init().unwrap();

    let mut time = Time::new();
    time.sync().unwrap();

    loop {
        sleep(Duration::from_millis(1000));
        if let Some(ref mut mq_client) = mq.client {
            let timestamps = time.get_time().as_millis();
            info!("timestamps {}", timestamps);
            mq_client.publish(
                "ups-0.2/heartbeat",
                QoS::AtMostOnce,
                false,
                timestamps.to_string().as_bytes(),
            ).map_err(|err| {
                warn!("publish heartbeat failed {}", err)
            }).unwrap();
        }
    }
}
